#!/usr/bin/env python3
"""
Пример плагина для Astresay. Первым аргументом для него должен выступать
путь к временному JSON-файлу с данными, полученными в процессе работы
диалога.
"""
import json
import sys

# Считываем файл с данными для плагина.
filename = sys.argv[1]
plugin_data = json.load(open(filename))

#######################
# Начало вашего кода. #
#######################


# Производим операции с ними.
print('Данные для плагина:', plugin_data)


######################
# Конец вашего кода. #
######################

# Обязательно указываем, что плагин успешно завершён, чтобы не прервать диалог.
sys.exit()
