# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stt_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
  name='stt_service.proto',
  package='yandex.cloud.ai.stt.v2',
  syntax='proto3',
  serialized_options=_b('Z>github.com/yandex-cloud/go-genproto/yandex/cloud/ai/stt/v2;stt'),
  serialized_pb=_b('\n\x11stt_service.proto\x12\x16yandex.cloud.ai.stt.v2\"\x88\x01\n\x1bStreamingRecognitionRequest\x12;\n\x06\x63onfig\x18\x01 \x01(\x0b\x32).yandex.cloud.ai.stt.v2.RecognitionConfigH\x00\x12\x17\n\raudio_content\x18\x02 \x01(\x0cH\x00\x42\x13\n\x11streaming_request\"f\n\x11RecognitionConfig\x12>\n\rspecification\x18\x01 \x01(\x0b\x32\'.yandex.cloud.ai.stt.v2.RecognitionSpec\x12\x11\n\tfolder_id\x18\x02 \x01(\t\"\xbf\x02\n\x0fRecognitionSpec\x12M\n\x0e\x61udio_encoding\x18\x01 \x01(\x0e\x32\x35.yandex.cloud.ai.stt.v2.RecognitionSpec.AudioEncoding\x12\x19\n\x11sample_rate_hertz\x18\x02 \x01(\x03\x12\x15\n\rlanguage_code\x18\x03 \x01(\t\x12\x18\n\x10profanity_filter\x18\x04 \x01(\x08\x12\r\n\x05model\x18\x05 \x01(\t\x12\x17\n\x0fpartial_results\x18\x07 \x01(\x08\x12\x18\n\x10single_utterance\x18\x08 \x01(\x08\"O\n\rAudioEncoding\x12\x1e\n\x1a\x41UDIO_ENCODING_UNSPECIFIED\x10\x00\x12\x10\n\x0cLINEAR16_PCM\x10\x01\x12\x0c\n\x08OGG_OPUS\x10\x02\"\x7f\n\x1cStreamingRecognitionResponse\x12>\n\x06\x63hunks\x18\x01 \x03(\x0b\x32..yandex.cloud.ai.stt.v2.SpeechRecognitionChunk\x12\x1f\n\x17\x65nd_of_single_utterance\x18\x02 \x01(\x08\"s\n\x16SpeechRecognitionChunk\x12J\n\x0c\x61lternatives\x18\x01 \x03(\x0b\x32\x34.yandex.cloud.ai.stt.v2.SpeechRecognitionAlternative\x12\r\n\x05\x66inal\x18\x02 \x01(\x08\"@\n\x1cSpeechRecognitionAlternative\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\x12\n\nconfidence\x18\x02 \x01(\x02\x32\x94\x01\n\nSttService\x12\x85\x01\n\x12StreamingRecognize\x12\x33.yandex.cloud.ai.stt.v2.StreamingRecognitionRequest\x1a\x34.yandex.cloud.ai.stt.v2.StreamingRecognitionResponse\"\x00(\x01\x30\x01\x42@Z>github.com/yandex-cloud/go-genproto/yandex/cloud/ai/stt/v2;sttb\x06proto3')
)



_RECOGNITIONSPEC_AUDIOENCODING = _descriptor.EnumDescriptor(
  name='AudioEncoding',
  full_name='yandex.cloud.ai.stt.v2.RecognitionSpec.AudioEncoding',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='AUDIO_ENCODING_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LINEAR16_PCM', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OGG_OPUS', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=529,
  serialized_end=608,
)
_sym_db.RegisterEnumDescriptor(_RECOGNITIONSPEC_AUDIOENCODING)


_STREAMINGRECOGNITIONREQUEST = _descriptor.Descriptor(
  name='StreamingRecognitionRequest',
  full_name='yandex.cloud.ai.stt.v2.StreamingRecognitionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='config', full_name='yandex.cloud.ai.stt.v2.StreamingRecognitionRequest.config', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='audio_content', full_name='yandex.cloud.ai.stt.v2.StreamingRecognitionRequest.audio_content', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='streaming_request', full_name='yandex.cloud.ai.stt.v2.StreamingRecognitionRequest.streaming_request',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=46,
  serialized_end=182,
)


_RECOGNITIONCONFIG = _descriptor.Descriptor(
  name='RecognitionConfig',
  full_name='yandex.cloud.ai.stt.v2.RecognitionConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='specification', full_name='yandex.cloud.ai.stt.v2.RecognitionConfig.specification', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='folder_id', full_name='yandex.cloud.ai.stt.v2.RecognitionConfig.folder_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=184,
  serialized_end=286,
)


_RECOGNITIONSPEC = _descriptor.Descriptor(
  name='RecognitionSpec',
  full_name='yandex.cloud.ai.stt.v2.RecognitionSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='audio_encoding', full_name='yandex.cloud.ai.stt.v2.RecognitionSpec.audio_encoding', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sample_rate_hertz', full_name='yandex.cloud.ai.stt.v2.RecognitionSpec.sample_rate_hertz', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='language_code', full_name='yandex.cloud.ai.stt.v2.RecognitionSpec.language_code', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='profanity_filter', full_name='yandex.cloud.ai.stt.v2.RecognitionSpec.profanity_filter', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='model', full_name='yandex.cloud.ai.stt.v2.RecognitionSpec.model', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='partial_results', full_name='yandex.cloud.ai.stt.v2.RecognitionSpec.partial_results', index=5,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='single_utterance', full_name='yandex.cloud.ai.stt.v2.RecognitionSpec.single_utterance', index=6,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _RECOGNITIONSPEC_AUDIOENCODING,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=289,
  serialized_end=608,
)


_STREAMINGRECOGNITIONRESPONSE = _descriptor.Descriptor(
  name='StreamingRecognitionResponse',
  full_name='yandex.cloud.ai.stt.v2.StreamingRecognitionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='chunks', full_name='yandex.cloud.ai.stt.v2.StreamingRecognitionResponse.chunks', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end_of_single_utterance', full_name='yandex.cloud.ai.stt.v2.StreamingRecognitionResponse.end_of_single_utterance', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=610,
  serialized_end=737,
)


_SPEECHRECOGNITIONCHUNK = _descriptor.Descriptor(
  name='SpeechRecognitionChunk',
  full_name='yandex.cloud.ai.stt.v2.SpeechRecognitionChunk',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='alternatives', full_name='yandex.cloud.ai.stt.v2.SpeechRecognitionChunk.alternatives', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='final', full_name='yandex.cloud.ai.stt.v2.SpeechRecognitionChunk.final', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=739,
  serialized_end=854,
)


_SPEECHRECOGNITIONALTERNATIVE = _descriptor.Descriptor(
  name='SpeechRecognitionAlternative',
  full_name='yandex.cloud.ai.stt.v2.SpeechRecognitionAlternative',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='text', full_name='yandex.cloud.ai.stt.v2.SpeechRecognitionAlternative.text', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='confidence', full_name='yandex.cloud.ai.stt.v2.SpeechRecognitionAlternative.confidence', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=856,
  serialized_end=920,
)

_STREAMINGRECOGNITIONREQUEST.fields_by_name['config'].message_type = _RECOGNITIONCONFIG
_STREAMINGRECOGNITIONREQUEST.oneofs_by_name['streaming_request'].fields.append(
  _STREAMINGRECOGNITIONREQUEST.fields_by_name['config'])
_STREAMINGRECOGNITIONREQUEST.fields_by_name['config'].containing_oneof = _STREAMINGRECOGNITIONREQUEST.oneofs_by_name['streaming_request']
_STREAMINGRECOGNITIONREQUEST.oneofs_by_name['streaming_request'].fields.append(
  _STREAMINGRECOGNITIONREQUEST.fields_by_name['audio_content'])
_STREAMINGRECOGNITIONREQUEST.fields_by_name['audio_content'].containing_oneof = _STREAMINGRECOGNITIONREQUEST.oneofs_by_name['streaming_request']
_RECOGNITIONCONFIG.fields_by_name['specification'].message_type = _RECOGNITIONSPEC
_RECOGNITIONSPEC.fields_by_name['audio_encoding'].enum_type = _RECOGNITIONSPEC_AUDIOENCODING
_RECOGNITIONSPEC_AUDIOENCODING.containing_type = _RECOGNITIONSPEC
_STREAMINGRECOGNITIONRESPONSE.fields_by_name['chunks'].message_type = _SPEECHRECOGNITIONCHUNK
_SPEECHRECOGNITIONCHUNK.fields_by_name['alternatives'].message_type = _SPEECHRECOGNITIONALTERNATIVE
DESCRIPTOR.message_types_by_name['StreamingRecognitionRequest'] = _STREAMINGRECOGNITIONREQUEST
DESCRIPTOR.message_types_by_name['RecognitionConfig'] = _RECOGNITIONCONFIG
DESCRIPTOR.message_types_by_name['RecognitionSpec'] = _RECOGNITIONSPEC
DESCRIPTOR.message_types_by_name['StreamingRecognitionResponse'] = _STREAMINGRECOGNITIONRESPONSE
DESCRIPTOR.message_types_by_name['SpeechRecognitionChunk'] = _SPEECHRECOGNITIONCHUNK
DESCRIPTOR.message_types_by_name['SpeechRecognitionAlternative'] = _SPEECHRECOGNITIONALTERNATIVE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StreamingRecognitionRequest = _reflection.GeneratedProtocolMessageType('StreamingRecognitionRequest', (_message.Message,), dict(
  DESCRIPTOR = _STREAMINGRECOGNITIONREQUEST,
  __module__ = 'stt_service_pb2'
  # @@protoc_insertion_point(class_scope:yandex.cloud.ai.stt.v2.StreamingRecognitionRequest)
  ))
_sym_db.RegisterMessage(StreamingRecognitionRequest)

RecognitionConfig = _reflection.GeneratedProtocolMessageType('RecognitionConfig', (_message.Message,), dict(
  DESCRIPTOR = _RECOGNITIONCONFIG,
  __module__ = 'stt_service_pb2'
  # @@protoc_insertion_point(class_scope:yandex.cloud.ai.stt.v2.RecognitionConfig)
  ))
_sym_db.RegisterMessage(RecognitionConfig)

RecognitionSpec = _reflection.GeneratedProtocolMessageType('RecognitionSpec', (_message.Message,), dict(
  DESCRIPTOR = _RECOGNITIONSPEC,
  __module__ = 'stt_service_pb2'
  # @@protoc_insertion_point(class_scope:yandex.cloud.ai.stt.v2.RecognitionSpec)
  ))
_sym_db.RegisterMessage(RecognitionSpec)

StreamingRecognitionResponse = _reflection.GeneratedProtocolMessageType('StreamingRecognitionResponse', (_message.Message,), dict(
  DESCRIPTOR = _STREAMINGRECOGNITIONRESPONSE,
  __module__ = 'stt_service_pb2'
  # @@protoc_insertion_point(class_scope:yandex.cloud.ai.stt.v2.StreamingRecognitionResponse)
  ))
_sym_db.RegisterMessage(StreamingRecognitionResponse)

SpeechRecognitionChunk = _reflection.GeneratedProtocolMessageType('SpeechRecognitionChunk', (_message.Message,), dict(
  DESCRIPTOR = _SPEECHRECOGNITIONCHUNK,
  __module__ = 'stt_service_pb2'
  # @@protoc_insertion_point(class_scope:yandex.cloud.ai.stt.v2.SpeechRecognitionChunk)
  ))
_sym_db.RegisterMessage(SpeechRecognitionChunk)

SpeechRecognitionAlternative = _reflection.GeneratedProtocolMessageType('SpeechRecognitionAlternative', (_message.Message,), dict(
  DESCRIPTOR = _SPEECHRECOGNITIONALTERNATIVE,
  __module__ = 'stt_service_pb2'
  # @@protoc_insertion_point(class_scope:yandex.cloud.ai.stt.v2.SpeechRecognitionAlternative)
  ))
_sym_db.RegisterMessage(SpeechRecognitionAlternative)


DESCRIPTOR._options = None

_STTSERVICE = _descriptor.ServiceDescriptor(
  name='SttService',
  full_name='yandex.cloud.ai.stt.v2.SttService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=923,
  serialized_end=1071,
  methods=[
  _descriptor.MethodDescriptor(
    name='StreamingRecognize',
    full_name='yandex.cloud.ai.stt.v2.SttService.StreamingRecognize',
    index=0,
    containing_service=None,
    input_type=_STREAMINGRECOGNITIONREQUEST,
    output_type=_STREAMINGRECOGNITIONRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_STTSERVICE)

DESCRIPTOR.services_by_name['SttService'] = _STTSERVICE

# @@protoc_insertion_point(module_scope)
